# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""OER to Alma workflow."""

from flask_principal import Identity
from invenio_alma import AlmaRESTService
from invenio_alma.services import AlmaRESTError
from invenio_alma.utils import is_duplicate_in_alma
from invenio_pidstore.errors import PIDDoesNotExistError
from invenio_records_lom import current_records_lom
from invenio_records_marc21 import Marc21Metadata, convert_json_to_marc21xml
from invenio_records_resources.services.records.results import RecordItem
from sqlalchemy.orm.exc import NoResultFound

from .convert import LOM2Marc21


def oer_create_in_alma_func(
    identity: Identity,
    lom_id: str,
    alma_service: AlmaRESTService,
) -> RecordItem:
    """Export OER to Alma."""
    lom_service = current_records_lom.records_service

    if is_duplicate_in_alma(lom_id):
        msg = f"WARNING: duplicat in alma lom_id: {lom_id}"
        raise RuntimeWarning(msg)

    try:
        record = lom_service.read(identity, lom_id)
    except (NoResultFound, PIDDoesNotExistError) as error:
        msg = f"ERROR: lom_id: {lom_id} not found in db"
        raise RuntimeError(msg) from error

    marc21_record = Marc21Metadata()
    lom_to_marc21 = LOM2Marc21()
    # TODO: check what record is, visit would need a dict
    lom_to_marc21.visit(record, marc21_record)

    marc21_record_etree = convert_json_to_marc21xml(marc21_record.json["metadata"])

    try:
        alma_service.create_record(marc21_record_etree)
    except AlmaRESTError as error:
        msg = f"ERROR: alma rest error on lom_id: {lom_id}, error: {error}"
        raise RuntimeError(msg) from error


def oer_create_aggregator() -> list[tuple[str, str]]:
    """OER create aggregator."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Open Access Workflows config."""

from invenio_records_resources.services.base.config import (
    ConfiguratorMixin,
    ServiceConfig,
)

from .api import WorkflowOpenaccess


class WorkflowOpenaccessServiceConfig(ServiceConfig, ConfiguratorMixin):
    """Workflow openaccess service config."""

    openaccess_cls = WorkflowOpenaccess



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Theses Workflows."""

from .config import WorkflowThesesServiceConfig
from .service import WorkflowThesesService
from .theses import (
    theses_create_aggregator,
    theses_create_func,
    theses_duplicate_func,
    theses_filter,
    theses_import_from_alma_func,
    theses_import_from_cms_func,
    theses_update_aggregator,
    theses_update_func,
)
from .views import create_blueprint

__all__ = (
    "WorkflowThesesService",
    "WorkflowThesesServiceConfig",
    "create_blueprint",
    "theses_create_aggregator",
    "theses_create_func",
    "theses_duplicate_func",
    "theses_filter",
    "theses_import_from_alma_func",
    "theses_import_from_cms_func",
    "theses_update_aggregator",
    "theses_update_func",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""API for theses workflow."""

from __future__ import annotations

from invenio_db import db

from .models import WorkflowOpenaccessMetadata


class WorkflowOpenaccess:
    """Workflow openaccess api."""

    model_cls = WorkflowOpenaccessMetadata

    def __init__(self, model: WorkflowOpenaccessMetadata) -> None:
        """Construct WorkflowOpenaccess."""
        self.model = model

    @property
    def pid(self) -> str:
        """Get pid."""
        return self.model.pid

    @property
    def pure_id(self) -> str:
        """Get pure id."""
        return self.model.pure_id

    @classmethod
    def resolve(cls, id_: str):  # noqa: ANN206
        """Get."""
        model = cls.model_cls.query.filter_by(pid=id_).one_or_none()
        return cls(model=model)

    @classmethod
    def get_ready_to(cls, state: str) -> list:
        """Get ready to."""
        entries = []
        if state == "marked_as_exported":
            entries = cls.model_cls.query.filter_by(
                imported_in_repo=True,
            )

        return [cls(model=entry) for entry in entries.all()]

    @classmethod
    def create(cls, id_: str, pure_id: str):  # noqa: ANN206
        """Create."""
        with db.session.begin_nested():
            entry = cls(model=cls.model_cls(pid=id_, pure_id=pure_id))
            db.session.add(entry.model)
        return entry

    def commit(self) -> None:
        """Commit."""
        with db.session.begin_nested():
            db.session.merge(self.model)

    def set_state(self, state: str, *, value: bool = True) -> None:
        """Set archived."""
        if state == "imported_in_repo":
            self.model.imported_in_repo = value
        if state == "marked_as_exported":
            self.model.marked_as_exported = value
        db.session.merge(self.model)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""CLI for workflows."""

from click import group

from .theses.cli import theses_group


@group()
def workflows() -> None:
    """Workflows cli."""


workflows.add_command(theses_group)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it
# and/or modify it under the terms of the MIT License; see LICENSE
# file for more details.

"""Converter Module to facilitate conversion of metadata."""

from contextlib import suppress

from invenio_records_lom.utils import LOMMetadata


def langstring(value: str, language: str = "x-none") -> dict:
    """Langstring."""
    return {
        "langstring": {
            "lang": language,
            "#text": value,
        },
    }


class Converter:
    """Converter base class."""

    def convert(self, parent: dict, record: LOMMetadata) -> None:
        """Convert method."""
        for attribute, value in parent.items():
            self.process(attribute, value, record)

    def process[T](self, attribute: str, value: T, record: LOMMetadata) -> None:
        """Execute the corresponding method to the attribute."""

        def func_not_found(*_: dict, **__: dict) -> None:
            msg = f"NO convert method for {attribute}"
            raise ValueError(msg)

        convert_func = getattr(self, f"convert_{attribute}", func_not_found)
        return convert_func(value, record)


class IMOOXToLOM(Converter):
    """Convert class to convert Mooc to LOM."""

    def __init__(self) -> None:
        """Construct MoocToLOM."""
        self.language = ""

    def set_language(self, parent: dict) -> None:
        """Set default language for langstring."""
        if "languages" not in parent["attributes"]:
            return

        if len(parent["attributes"]["languages"]) == 0:
            return

        self.language = parent["attributes"]["languages"][0]

    def append_contribute(
        self,
        contributors: list,
        record: LOMMetadata,
        default_role: str = "",
    ) -> None:
        """Append to contribute."""
        for contributor in contributors:
            try:
                description = contributor["description"]
            except KeyError:
                description = None

            try:
                role = contributor["type"]
            except KeyError:
                role = default_role

            record.append_contribute(
                name=contributor["name"],
                role=role,
                description=description,
            )

    def convert(self, parent: dict, record: LOMMetadata) -> None:
        """Convert overrides base convert method to return the record."""
        self.set_language(parent)

        super().convert(parent, record)

    def convert_id(self, value: str, record: LOMMetadata) -> None:
        """Convert id attribute."""
        record.append_identifier(id_=value, catalog="imoox")

    def convert_courseCode(self, value: str, record: LOMMetadata) -> None:
        """Convert courseCode attribute."""
        record.append_identifier(id_=value, catalog="imoox")

    def convert_attributes(self, value: dict, record: LOMMetadata) -> None:
        """Convert attributes attribute."""
        super().convert(value, record)

    def convert_name(self, value: str, record: LOMMetadata) -> None:
        """Convert name attribute."""
        record.set_title(title=value, language_code=self.language)

    def convert_abstract(self, value: str, record: LOMMetadata) -> None:
        """Convert abstract attribute."""
        record.append_description(description=value, language_code=self.language)

    def convert_description(self, value: str, record: LOMMetadata) -> None:
        """Convert description attribute."""
        record.append_description(description=value, language_code=self.language)

    def convert_inLanguage(self, value: list, record: LOMMetadata) -> None:
        """Convert language attribute."""
        for lang in value:
            record.append_language(lang)

    def convert_startDate(self, value: str, record: LOMMetadata) -> None:
        """Convert startDate attribute."""
        record.set_datetime(value[0].split("T")[0])

    def convert_image(self, value: dict, record: LOMMetadata) -> None:
        """Convert image attribute."""
        record.set_thumbnail(value)

    def convert_instructor(self, value: list[dict], record: LOMMetadata) -> None:
        """Convert instructor attribute."""
        self.append_contribute(value, record, default_role="Author")

    def convert_learningobjectives(self, value: str, record: LOMMetadata) -> None:
        """Convert learningobjectives attribute."""
        for desc in value:
            record.append_educational_desription(
                description=desc,
                language_code=self.langugage,
            )

    def convert_duration(self, value: str, record: LOMMetadata) -> None:
        """Convert duration attribute."""
        record.set_duration(value=value, language=self.language)

    def convert_partnerInstitute(self, value: list[dict], record: LOMMetadata) -> None:
        """Convert partnerInstitute attribute."""
        self.append_contribute(value, record, default_role="Publisher")

    def convert_moocProvider(self, value: dict, record: LOMMetadata) -> None:
        """Convert moocProvider attribute."""
        record.append_metametadata_contribute(
            name=value["name"],
            url="",
            logo="",
            role="Provider",
        )

    def convert_url(self, value: str, record: LOMMetadata) -> None:
        """Convert url attribute."""
        record.set_location(value)

    def convert_workload(self, value: str, record: LOMMetadata) -> None:
        """Convert workload attribute."""
        record.set_typical_learning_time(value, description="workload")

    def convert_courseLicenses(self, value: list, record: LOMMetadata) -> None:
        """Convert courseLicenses attribute."""
        record.set_rights_url(url=value[0]["url"])

    def convert_license(self, value: list, record: LOMMetadata) -> None:
        """Convert courseLicense attribute."""
        record.set_rights_url(url=value[0]["url"])

    def convert_categories(self, value: dict, record: LOMMetadata) -> None:
        """Convert categories attribute."""
        super().convert(value, record)

    def convert_contributor(self, value: list[dict], record: LOMMetadata) -> None:
        """Convert contributor."""
        self.append_contribute(value, record)

    def convert_publisher(self, value: dict, record: LOMMetadata) -> None:
        """Convert publisher."""
        self.append_contribute([value], record)

    def convert_educationalAlignment(
        self,
        value: list[dict],
        record: LOMMetadata,
    ) -> None:
        """Convert educationalAlignment."""
        for item in value:
            for subitem in item["name"]:
                with suppress(KeyError):
                    record.append_oefos_id(
                        oefos_id=item["shortCode"],
                        language_code=subitem["inLanguage"],
                    )

    def convert_creator(self, value: list[dict], record: LOMMetadata) -> None:
        """Convert creator."""
        self.append_contribute(value, record)

    ####
    #
    # those categories are not mapped to lom
    #
    ####

    def convert_type(self, value: str, record: LOMMetadata) -> None:
        """Convert type attribute."""

    def convert_courseMode(self, value: dict[list], record: LOMMetadata) -> None:
        """Convert courseMode attribute."""

    def convert_availableUntil(self, value: str, record: LOMMetadata) -> None:
        """Convert availableUntil attribute."""

    def convert_endDate(self, value: str, record: LOMMetadata) -> None:
        """Convert endDate attribute."""
        # see startDate

    def convert_video(self, value: str, record: LOMMetadata) -> None:
        """Convert video attribute."""

    def convert_access(self, value: dict[list], record: LOMMetadata) -> None:
        """Convert access attribute."""

    def convert_learningResourceType(self, value: dict, record: LOMMetadata) -> None:
        """Convert learningResourceType."""
        learningresourcetype: str = value.get(
            "identifier",
            "https://w3id.org/kim/hcrt/course",
        )
        record.append_learningresourcetype(learningresourcetype)

    def convert_expires(self, value: list[str], record: LOMMetadata) -> None:
        """Convert expires."""

    def convert_trailer(self, value: dict, record: LOMMetadata) -> None:
        """Convert trailer."""

    def convert_teaches(self, value: list[dict], record: LOMMetadata) -> None:
        """Convert teaches."""

    def convert_audience(self, value: list[str], record: LOMMetadata) -> None:
        """Convert audience."""

    def convert_educationalLevel(self, value: list[dict], record: LOMMetadata) -> None:
        """Convert educationallevel."""

    def convert_keywords(self, value: list[str], record: LOMMetadata) -> None:
        """Convert keywords."""

    def convert_contentLocation(self, value: dict, record: LOMMetadata) -> None:
        """Convert contentlocation."""

    def convert_offer(self, value: list, record: LOMMetadata) -> None:
        """Convert offer."""

    def convert_numberOfCredits(self, value: list, record: LOMMetadata) -> None:
        """Convert numberOfCredits."""

    def convert_educationalCredentialsAwarded(
        self,
        value: list[str],
        record: LOMMetadata,
    ) -> None:
        """Convert educationalCredentialsAwarded."""

    def convert_competencyRequired(
        self,
        value: list[dict],
        record: LOMMetadata,
    ) -> None:
        """Convert competencyRequired."""

    def convert_applicationStartDate(self, value: str, record: LOMMetadata) -> None:
        """Convert applicationStartDate."""

    def convert_applicationDeadline(self, value: str, record: LOMMetadata) -> None:
        """Convert applicationDeadline."""

    def convert_accessMode(self, value: list[str], record: LOMMetadata) -> None:
        """Convert accessMode."""

    def convert_repeatFrequency(self, value: list[dict], record: LOMMetadata) -> None:
        """Convert repeatFrequency."""

    def convert_dateCreated(self, value: str, record: LOMMetadata) -> None:
        """Convert dateCreated."""

    def convert_dateModified(self, value: str, record: LOMMetadata) -> None:
        """Convert dateModified."""

    def convert_hollandCode(self, value: list[str], record: LOMMetadata) -> None:
        """Convert hollandCode."""

    def convert_hasPart(self, value: list[dict], record: LOMMetadata) -> None:
        """Convert hasPart."""

    def convert_isPartOf(self, value: list[dict], record: LOMMetadata) -> None:
        """Convert isPartOf."""



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Teachcenter workflows."""


from pathlib import Path

from flask_principal import Identity
from invenio_moodle import MoodleRESTService
from invenio_pidstore.errors import PIDDoesNotExistError
from invenio_pidstore.models import PersistentIdentifier
from invenio_records_lom.proxies import current_records_lom
from invenio_records_lom.services import LOMRecordService
from invenio_records_lom.utils import LOMRecordData, create_record, update_record
from invenio_records_resources.services.records.results import RecordItem

from .types import BaseRecord, FileKey, FileRecord, Key, LinkKey, LinkRecord, Status
from .visitor import TeachCenterToLOM


def create_key(tc_record: dict) -> Key:
    """Create key."""
    # TODO implement LinkKey (opencast)
    return FileKey(tc_record)


def create_draft(
    identity: Identity,
    key: Key,
    moodle_pid_value: str,
    records_service: LOMRecordService,
) -> RecordItem:
    """Create draft with empty metadata."""
    pids = {
        "moodle": {
            "provider": "moodle",
            "identifier": moodle_pid_value,
        },
    }
    data = LOMRecordData(resource_type=key.resource_type, pids=pids)
    data.metadata.append_identifier(moodle_pid_value, catalog="moodle")
    return records_service.create(data=data.json, identity=identity)


def get_from_database_or_create(
    identity: Identity,
    key: Key,
    records_service: LOMRecordService,
) -> BaseRecord:
    """Fetch moodle-result corresponding to `key`, create database-entry if none exists.

    :param Key key: the key which to attempt fetching from pidstore


    File:
    A File could be already in database. A File should not be added twice and it
    is looked up by its file hash (hash_sha1). If it is there already it will be
    linked to the new Unit where it is used.

    """
    moodle_pid_value = key.get_moodle_pid_value()

    try:
        moodle_pid = PersistentIdentifier.get(
            pid_type="moodle",
            pid_value=moodle_pid_value,
        )
    except PIDDoesNotExistError:
        draft = create_draft(identity, key, moodle_pid_value, records_service)
        pid = draft.id
        data = LOMRecordData(**draft.to_dict())
        status = Status.NEW
    else:
        # get lomid corresponding to moodle_pid
        lom_pid = PersistentIdentifier.get_by_object(
            pid_type="lomid",
            object_type=moodle_pid.object_type,
            object_uuid=moodle_pid.object_uuid,
        )

        pid = lom_pid.pid_value
        draft = records_service.edit(id_=pid, identity=identity)
        data = LOMRecordData(**draft.to_dict())
        status = Status.EDIT

    match key:
        case FileKey():
            type_of_record = FileRecord
        case LinkKey():
            type_of_record = LinkRecord

    return type_of_record(key, pid, data, status, draft)


def is_duplicate(draft: RecordItem, new_course_ids: list[str]) -> bool:
    """Check about duplicate."""
    try:
        existing_course_ids = [
            identifier["entry"]["langstring"]["#text"]
            for course in draft["metadata"]["courses"]
            for identifier in course["course"]["identifier"]
        ]
    except KeyError:
        existing_course_ids = []

    return any(id_ in existing_course_ids for id_ in new_course_ids)


def teachcenter_import_func(  # noqa: C901
    identity: Identity,
    tc_record: dict,
    moodle_service: MoodleRESTService,
    *,
    dry_run: bool = False,
) -> None:
    """Insert data encoded in `moodle-data` into invenio-database.

    :param dict moodle_data: The data to be inserted into database,
        whose format matches `MoodleSchema`
    :param Identity identity
    """
    records_service = current_records_lom.records_service

    record_key = create_key(tc_record)
    draft = get_from_database_or_create(identity, record_key, records_service)

    visitor = TeachCenterToLOM()
    visitor.visit(tc_record, draft.data.metadata)

    file_url = visitor.file_url

    file_paths = []
    if isinstance(draft, FileRecord) and draft.status == Status.NEW:
        file_paths += [moodle_service.download_file(identity, file_url)]

    if dry_run:
        if draft.status == Status.NEW:
            records_service.delete_draft(id_=draft.draft.id, identity=identity)
        if len(file_paths):
            for file_path in file_paths:
                Path(file_path).unlink()
        msg = f"DRY_RUN teachcenter import success id: {record_key}"
        raise RuntimeError(msg)

    match draft.status:
        case Status.NEW:
            data = draft.data
            record = create_record(records_service, data.json, file_paths, identity)
        case Status.EDIT:
            if is_duplicate(draft.draft, visitor.course_ids):
                msg = f"WARNING course already in record pid: {draft.pid}"
                raise RuntimeError(msg)

            data = draft.draft.data
            for course in draft.data.metadata.get_courses():
                if course not in data["metadata"]["courses"]:
                    data["metadata"]["courses"].append(course)

            record = update_record(draft.pid, records_service, data, identity)

    return record



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Teachcenter configs."""

from .workflows import teachcenter_import_func

__all__ = ("teachcenter_import_func",)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Convert from LOM to Marc21."""

from datetime import datetime

from invenio_records_marc21.services.record.metadata import Marc21Metadata


class Visitor:
    """Visitor base class."""

    def visit(self, value: dict, record: Marc21Metadata) -> None:
        """Convert record from LOM JSON format to MARC21XML."""
        for attribute, val in value.items():
            self.visit_attribute(attribute, val, record)

    def visit_attribute(
        self,
        attribute: str,
        value: dict,
        record: Marc21Metadata,
    ) -> None:
        """Traverse first level elements of dictionary and extract attributes."""

        def func_not_found(*_: dict, **__: dict) -> None:
            msg = f"NO visitor node: '{attribute}'"
            raise ValueError(msg)

        visit_function = getattr(self, f"visit_{attribute}", func_not_found)
        return visit_function(value, record)


class LOM2Marc21(Visitor):
    """LOM to Marc21 converter class."""

    def __init__(self, record: Marc21Metadata, doi: str) -> None:
        """Construct."""
        super().__init__()

        record.emplace_leader("07878nai a2200421 c 4500")
        record.emplace_controlfield("007", "ccr|||||||||||")
        record.emplace_datafield("024.7.0.", subfs={"a": doi, "2": "doi"})
        record.emplace_datafield("040.0.0.", subfs={"b": "ger", "e": "rda"})
        record.emplace_datafield("044.0.0.", subfs={"c": "XA-AT"})
        record.emplace_datafield("336.0.0.", subfs={"b": "txt"})
        record.add_datafield("336.0.0.", subfs={"b": "tdi"})
        record.emplace_datafield("337.0.0.", subfs={"b": "c"})
        record.emplace_datafield("338.0.0.", subfs={"b": "cr"})
        record.emplace_datafield("347.0.0.", subfs={"a": "Textdatei"})
        record.add_datafield("347.0.0.", subfs={"a": "Videodatei"})
        record.emplace_datafield("500.0.0.", subfs={"a": "Gesehen am"})
        record.emplace_datafield(
            "506.0.0.",
            subfs={"f": "Unrestricted online access", "2": "star"},
        )
        record.emplace_datafield(
            "655.0.7.",
            subfs={
                "a": "Massive Open Online Course",
                "0": "$$0 (DE-588)107372798X",
                "2": "gnd-content",
            },
        )
        record.emplace_datafield(
            "995...",
            subfs={"i": "TU Graz Repository", "a": doi, "9": "local"},
        )

        self.title = ""
        self.entities = []

    def visit(self, value: dict, record: Marc21Metadata) -> None:
        """Visit."""
        super().visit(value, record)

        if self.title == "":
            return

        subfs = {"a": self.title}

        if len(self.entities) > 0:
            subfs["c"] = ",".join(self.entities[0]["name"])

        record.emplace_datafield(
            "245.0.0.",
            subfs=subfs,
        )

    def visit_general(self, value: dict, record: Marc21Metadata) -> None:
        """Process General."""
        super().visit(value, record)

    def visit_title(self, value: dict, _: Marc21Metadata) -> None:
        """Process ."""
        self.title = value["langstring"]["#text"]

    def visit_language(self, value: str, record: Marc21Metadata) -> None:
        """Process ."""
        for language in value:
            record.add_datafield("041.0.0.", subfs={"a": language})

    def visit_description(self, value: list, record: Marc21Metadata) -> None:
        """Process ."""
        for description in value:
            text = description["langstring"]["#text"]
            record.add_datafield("520.0.0.", subfs={"a": text})

    def visit_educational_description(
        self,
        value: list,
        record: Marc21Metadata,
    ) -> None:
        """Process ."""
        for description in value:
            text = description["langstring"]["#text"]
            record.add_datafield("521.0.0.", subfs={"a": text})

    def visit_technical(self, value: dict, record: Marc21Metadata) -> None:
        """Process technical."""
        super().visit(value, record)

    def visit_thumbnail(self, value: dict, record: Marc21Metadata) -> None:
        """Process thumbnail."""
        super().visit(value, record)

    def visit_licenses(self, value: dict, record: Marc21Metadata) -> None:
        """Process licenses."""
        try:
            id_ = value["id"]
        except KeyError:
            id_ = None

        try:
            url = value["url"]
        except KeyError:
            url = None

        if id_ is None or url is None:
            return

        record.emplace_datafield(
            "540.0.0.",
            subfs={
                "2": "cc",
                "a": "Thumbnail",
                "f": id_,
                "u": url,
            },
        )

    def visit_location(self, value: dict, record: Marc21Metadata) -> None:
        """Process location."""
        text = value["#text"]
        record.emplace_datafield(
            "856.4.0.",
            subfs={"u": text, "x": "IMOOX", "z": "kostenfrei", "3": "Volltext"},
        )

    def visit_duration(self, value: dict, record: Marc21Metadata) -> None:
        """Process duration."""
        # TODO nachfragen
        # record.emplace_datafield("300.0.0.", subfs={"a": "Online-Ressource"})

    def visit_educational(self, value: dict, record: Marc21Metadata) -> None:
        """Process educational."""
        if "description" in value:
            self.visit_educational_description(value["description"], record)
            del value["description"]

        super().visit(value, record)

    def visit_rights(self, value: dict, record: Marc21Metadata) -> None:
        """Process rights."""
        super().visit(value, record)

    def visit_url(self, value: str, record: Marc21Metadata) -> None:
        """Process url."""
        record.emplace_datafield("540.0.0.", subfs={"a": "MOOC", "u": value, "2": "cc"})

    def visit_classification(self, value: dict, record: Marc21Metadata) -> None:
        """Process classification."""
        super().visit(value, record)

    def visit_lifecycle(self, value: dict, record: Marc21Metadata) -> None:
        """Process lifecycle."""
        super().visit(value, record)

    def visit_datetime(self, value: str, record: Marc21Metadata) -> None:
        """Process datetime."""
        dt = datetime.strptime(value, "%Y-%m-%d %H:%M:%S")
        startdate = value
        record.emplace_datafield(
            "264.3.1.",
            subfs={"a": "Graz", "b": "Technische Universität Graz", "c": startdate},
        )
        record.emplace_controlfield(
            "008",
            f"230501c{dt.year}9999   #####o#####|||#|2### #",
        )
        # TODO empty_expected

    def visit_contribute(self, value: list, record: Marc21Metadata) -> None:
        """Process contribute."""
        for contribute in value:
            description = contribute["date"]["description"]["langstring"]["#text"]
            role = contribute["role"]["value"]["langstring"]["#text"]
            entities = contribute["entity"]

            for entity in entities:
                self.entities += [{"name": entity, "role": role}]
            record.emplace_datafield("545.0.0.", subfs={"a": description})

        for idx, x in enumerate(self.entities):
            if idx == 0:
                record.emplace_datafield("100.1.0.", subfs={"a": x["name"], "4": "aut"})
            else:
                record.emplace_datafield(
                    "700.1.0.",
                    subfs={
                        "a": x["name"],
                        "4": x["role"],
                    },
                )



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Open Access Workflow."""


from flask_principal import Identity
from invenio_access.permissions import system_identity
from invenio_pidstore.errors import PIDDoesNotExistError
from invenio_pidstore.models import PersistentIdentifier
from invenio_pure import PureRuntimeError
from invenio_pure.records.models import PureRESTError
from invenio_pure.services import PureRESTService
from invenio_pure.types import PureID
from invenio_records_marc21 import (
    Marc21Metadata,
    add_file_to_record,
    create_record,
    current_records_marc21,
)
from invenio_records_marc21.records import Marc21Draft, Marc21Record
from invenio_records_resources.services.records.results import RecordItem
from marshmallow.exceptions import ValidationError
from sqlalchemy.exc import IntegrityError, NoResultFound
from sqlalchemy.orm.exc import StaleDataError

from ..proxies import current_workflows_tugraz
from .convert import Pure2Marc21
from .utils import change_to_exported, extract_files


def openaccess_filter() -> dict:
    """Openaccess filter."""
    return {
        "keywordUris": ["dk/atira/pure/researchoutput/keywords/export2repo/validated"],
    }


def openaccess_mark_as_exported_aggregator() -> list[tuple[str, str]]:
    """Return a list of tuple[marc21, pure_id] which should be marked as exported in pure."""
    oa_service = current_workflows_tugraz.openaccess_service
    return oa_service.get_ready_to(system_identity, state="imported_in_repo")


def openaccess_import_func(  # noqa: PLR0915
    identity: Identity,
    pure_id: PureID,
    pure_service: PureRESTService,
) -> RecordItem:
    """Import record from pure into the repository."""
    marc21_service = current_records_marc21.records_service
    oa_service = current_workflows_tugraz.openaccess_service
    ignore_files = False

    try:
        # resolve record over pure_id
        pid = PersistentIdentifier.get("pure", pure_id)
        obj = pid.get_assigned_object(object_type="rec")

        try:
            draft_model = Marc21Draft.get_record(obj)
            # the edit is not necessary, but to get a resultitem draft this is the easiest way
            draft = marc21_service.edit(identity=identity, id_=draft_model["id"])
        except NoResultFound:
            record_model = Marc21Record.get_record(obj)
            draft = marc21_service.edit(identity=identity, id_=record_model["id"])
        ignore_files = True
    except PIDDoesNotExistError:
        # not found create a record
        data = {
            "files": {"enabled": True},
            "access": {
                "record": "public",
                "files": "public",
            },
        }
        draft = create_record(
            marc21_service,
            data=data,
            file_paths=[],
            identity=identity,
            do_publish=False,
        )

    try:
        pure_record = pure_service.get_metadata(identity, pure_id)
        files = extract_files(pure_record)
        file_paths = [pure_service.download_file(identity, file_) for file_ in files]
    except (PureRESTError, PureRuntimeError) as error:
        # todo: delete draft
        draft.delete_draft(identity=identity, id_=draft.id)
        raise RuntimeError(str(error)) from error

    marc21_record = Marc21Metadata()
    converter = Pure2Marc21()

    try:
        converter.convert(pure_record, marc21_record)
    except KeyError as error:
        raise RuntimeError(str(error)) from error

    # the publisher is not included in the record information
    publisher = pure_service.get_publisher_name(identity, pure_id)
    marc21_record.emplace_datafield("264..1.b", value=publisher)

    # TODO merge with draft.data
    # find out how to get data from draft
    data = draft.to_dict() | marc21_record.json
    data["pids"]["pure"] = {"provider": "pure", "identifier": pure_id}

    try:
        marc21_service.update_draft(identity, id_=draft.id, data=data)

        # TODO:
        # there exists a problem with reupload files, this is the easiest solution for the moment
        if not ignore_files:
            for file_path in file_paths:
                add_file_to_record(
                    marcid=draft.id,
                    file_path=file_path,
                    file_service=marc21_service.draft_files,
                    identity=identity,
                )

        # validate the draft here so that the record could be marked as exported
        # because the publish will go through without problems
        marc21_service.validate_draft(
            identity,
            id_=draft.id,
            ignore_field_permissions=True,
        )
    except StaleDataError as error:
        msg = f"ERROR: PureImport StaleDataError pure_id: {pure_id}"
        raise RuntimeError(msg) from error
    except ValidationError as error:
        msg = f"ERROR: PureImport ValidationError pure_id: {pure_id}, error: {error}"
        raise RuntimeError(msg) from error

    # since the valid import has been checked directly after creating the draft
    # the publish should work without errors.
    record = marc21_service.publish(id_=draft.id, identity=identity)

    try:
        oa_service.create(identity, record.id, pure_id)
    except IntegrityError:
        # if a record will be reimported after resetting the
        # ready-to-export tag in pure. the record in pure has to be
        # updated again, to make that happen the marked_as_exported
        # has to be resetted
        oa_service.set_state(
            identity,
            id_=record.id,
            state="marked_as_exported",
            value=False,
        )

    oa_service.set_state(identity, id_=record.id, state="imported_in_repo")

    return record


def openaccess_update_status_in_pure(
    identity: Identity,
    marc_id: str,
    pure_id: PureID,
    pure_service: PureRESTService,
) -> None:
    """Update status in pure."""
    oa_service = current_workflows_tugraz.openaccess_service

    try:
        # yes i know this is not nice to nicest way to get the xml record, but
        # the easiest
        pure_record = pure_service.get_metadata(identity, pure_id)
    except (PureRESTError, PureRuntimeError) as error:
        raise RuntimeError(str(error)) from error

    try:
        change_to_exported(pure_record)
        pure_service.mark_as_exported(identity, pure_id, pure_record)
    except PureRESTError as error:
        raise RuntimeError(str(error)) from error

    oa_service.set_state(identity, id_=marc_id, state="marked_as_exported")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Convert from CampusOnline to Marc21."""

from datetime import datetime
from operator import itemgetter
from re import split
from xml.etree.ElementTree import Element

from invenio_records_marc21.services.record.metadata import Marc21Metadata, QName


def construct_name(name: dict[str, str]) -> str:
    """Construct name."""
    return f"{name['ln']}, {name['fn']}"


def language_decode(lang: str) -> str:
    """Language decode."""
    languages = {
        "DE": "ger",
        "EN": "eng",
    }
    return languages.get(lang, "PLATZHALTER")


class Visitor:
    """Visitor base class."""

    def process(self, node: Element, record: Marc21Metadata) -> None:
        """Execute the corresponding method to the tag name."""

        def func_not_found(*_: dict, **__: dict) -> None:
            localname = QName(node).localname
            namespace = QName(node).namespace
            msg = f"NO visitor node: '{localname}' ns: '{namespace}'"
            raise ValueError(msg)

        tag_name = QName(node).localname
        visit_func = getattr(self, f"visit_{tag_name}", func_not_found)
        return visit_func(node, record)

    def visit(self, node: Element, record: Marc21Metadata) -> None:
        """Entry point for visitor."""
        for child in node:
            self.process(child, record)

    def visit_attr(self, node: Element, record: Marc21Metadata) -> None:
        """Run attr function."""
        key = node.attrib["key"]

        def func_not_found(*_: dict, **__: dict) -> None:
            msg = f"NO visitor node: '{key}'"
            raise ValueError(msg)

        visit_func = getattr(self, f"visit_{key}", func_not_found)
        visit_func(node, record)

    def convert(self, node: Element, record: Marc21Metadata) -> None:
        """Convert."""
        self.visit(node, record)


class ThesesLocalField:
    """Class to handle the various local fields."""

    def __init__(self) -> None:
        """Construct class."""
        self.theses_local_field = {}

    def add(self, key: str, value: dict) -> None:
        """Add key value pair."""
        if key not in self.theses_local_field:
            self.theses_local_field[key] = []

        if value not in self.theses_local_field[key]:
            self.theses_local_field[key].append(value)

    def items(self) -> tuple:
        """Return list of tuples."""
        for key, values in self.theses_local_field.items():
            for subfs in values:
                yield (key, subfs)


class CampusOnlineToMarc21(Visitor):
    """Convertor from CampusOnline to Marc21."""

    def __init__(self, record: Marc21Metadata) -> None:
        """Construct."""
        super().__init__()
        self.author_name = "N/A"
        self.state = ""
        self.metaclass_name = ""
        self.theses_local_field = ThesesLocalField()

        record.emplace_leader("07878nam a2200421 c 4500")
        record.emplace_controlfield("007", "cr#|||||||||||")
        record.emplace_controlfield("008", "230501s????####   #####om####|||#|#### c")
        record.emplace_datafield(
            "040...",
            subfs={"a": "AT-UBTUG", "b": "ger", "d": "AT-UBTUG", "e": "rda"},
        )
        record.emplace_datafield("044...", subfs={"c": "XA-AT"})
        record.emplace_datafield("264..1.", subfs={"a": "Graz", "c": "DATUM"})
        record.emplace_datafield(
            "300...",
            subfs={"a": "1 Online-Ressource (Seiten)", "b": "ill"},
        )
        record.emplace_datafield("336...", subfs={"b": "txt"})
        record.emplace_datafield("337...", subfs={"b": "c"})
        record.emplace_datafield("338...", subfs={"b": "cr"})
        record.emplace_datafield("347...", subfs={"a": "Textdatei", "b": "PDF"})
        record.emplace_datafield(
            "506.0..",
            subfs={"2": "star", "f": "Unrestricted online access"},
        )
        record.emplace_datafield("546...", subfs={"a": "Zusammenfassung in"})
        record.emplace_datafield(
            "655..7.",
            subfs={
                "a": "Hochschulschrift",
                "0": "(DE-588)4113937-9",
                "2": "gnd-content",
            },
        )

    def convert(self, node: Element, record: Marc21Metadata) -> None:
        """Override convert."""
        super().convert(node, record)
        for key, subfs in sorted(self.theses_local_field.items(), key=itemgetter(0)):
            record.add_datafield(key, subfs=subfs)

    def visit_ID(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ID."""
        record.emplace_datafield(
            "995...",
            subfs={"i": "TUGRAZonline", "a": str(node.text), "9": "local"},
        )

    def visit_PAG(self, node: Element, record: Marc21Metadata) -> None:
        """Visit PAG."""

    def visit_CO(self, node: Element, record: Marc21Metadata) -> None:
        """Visit CO."""

    def visit_CHD(self, node: Element, record: Marc21Metadata) -> None:
        """Visit CHD."""

    def visit_EJAHR(self, node: Element, record: Marc21Metadata) -> None:
        """Visit EJAHR."""

    def visit_ARCHD(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ARCHD."""

    def visit_PUBD(self, node: Element, record: Marc21Metadata) -> None:
        """Visit PUBD."""

    def visit_PUBLIC(self, node: Element, record: Marc21Metadata) -> None:
        """Visit PUBLIC."""

    def visit_STATUS(self, node: Element, record: Marc21Metadata) -> None:
        """Visit status."""

    def visit_STATUSD(self, node: Element, _: Marc21Metadata) -> None:
        """Visit Status date."""
        try:
            year = datetime.strptime(node.text, "%Y-%m-%d %H:%M:%S").year
        except ValueError:
            year = "JAHR"
        self.year = str(year)

    def visit_ORG(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_ORGP(self, node: Element, _: Marc21Metadata) -> None:
        """Visit ."""
        orgp = node.text.split("&gt;")

        faculty = orgp[1] if len(orgp) > 1 else ""
        institute = orgp[2] if len(orgp) > 2 else ""  # noqa: PLR2004

        self.theses_local_field.add(
            "971.5..",
            {
                "a": "Technische Universität Graz",
                "b": faculty.strip(),
                "c": institute.strip(),
                "d": "NUMMER",
            },
        )

    def visit_TYPKB(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""
        value = "HS-DISS" if node.text == "DISS" else "HS-MASTER"
        record.emplace_datafield("970.2..", subfs={"d": value})

    def visit_TYP(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""
        if self.state == "metaobj":
            self.typ = node.text
            return

        record.emplace_datafield(
            "502...",
            subfs={
                "b": node.text,
                "c": "Technische Universität Graz",
                "d": str(self.year),
            },
        )

    def visit_ZUGKB(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_ZUG(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_SPSTAT(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_SPVON(self, node: Element, _: Marc21Metadata) -> None:
        """Visit ."""
        text = node.text
        self.spvon = text if text else ""

    def visit_SPBIS(self, node: Element, _: Marc21Metadata) -> None:
        """Visit ."""
        text = node.text

        if not text:
            return

        in_format = "%Y-%m-%d %H:%M:%S"
        out_format = "%d.%m.%Y"

        spvon = datetime.strptime(self.spvon, in_format).strftime(out_format)
        spbis = datetime.strptime(text, in_format).strftime(out_format)

        self.theses_local_field.add(
            "971.7..",
            {"a": "gesperrt", "b": spvon, "c": spbis},
        )

    def visit_SPBGR(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_OLANG(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""
        self.object_language = node.text
        record.emplace_datafield("041...a", value=language_decode(node.text))

    def visit_TLANGS(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_VOLLTEXT(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_metaclass(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""
        self.visit(node, record)

    def visit_name(self, node: Element, _: Marc21Metadata) -> None:
        """Visit ."""
        self.metaclass_name = node.text

    def visit_metaobj(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""
        if self.metaclass_name not in ["AUTHOR", "TEXT", "SUPERVISOR"]:
            return

        self.state = "metaobj"
        self.name = {"fn": "", "ln": ""}

        self.visit(node, record)

        if self.metaclass_name == "AUTHOR":
            self.author_name = self.name
            record.emplace_datafield(
                "100.1..",
                subfs={"a": construct_name(self.author_name), "4": "aut"},
            )

        if self.metaclass_name == "SUPERVISOR":
            types = {
                "BTEXT": "0",
                "BTTUG": "0",
                "MBTUG": "2",
                "MBEXT": "2",
                "1BUTUG": "1",
                "2BUTUG": "1",
                "3BUTUG": "1",
                "4BUTUG": "1",
                "5BUTUG": "1",
                "6BUTUG": "1",
            }
            ind1 = types.get(self.typ, "")
            self.theses_local_field.add(
                f"971.{ind1}..",
                {"a": construct_name(self.name)},
            )

        self.state = ""

    def visit_FN(self, node: Element, _: Marc21Metadata) -> None:
        """Visit ."""
        self.name["fn"] = node.text

    def visit_LN(self, node: Element, _: Marc21Metadata) -> None:
        """Visit ."""
        self.name["ln"] = node.text

    def visit_FNLN(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_AKK(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_MNR(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_STRI(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_STKZ(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_ORIG(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_INTERN(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""

    def visit_TIT(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""
        text = node.text.replace("\n", " ")
        author = f"{self.author_name['fn']} {self.author_name['ln']}"

        if self.state == "metaobj" and self.language == self.object_language:
            record.emplace_datafield("245.1.0.", subfs={"a": text, "c": author})

        if self.state == "metaobj" and self.language != self.object_language:
            record.emplace_datafield("246.1..", subfs={"i": "TUGRAZonline", "a": text})

    def visit_ABS(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""
        record.add_datafield("520...", value=node.text)

    def visit_KEYW(self, node: Element, record: Marc21Metadata) -> None:
        """Visit ."""
        text = node.text
        if not text:
            return

        subjects = filter(
            lambda s: len(s) > 0,
            [s.strip() for s in split(r";|,", text)],
        )

        for subject in subjects:
            record.add_datafield("653...", value=subject)

    def visit_LANG(self, node: Element, _: Marc21Metadata) -> None:
        """Visit ."""
        self.language = node.text



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Initialize the package."""

from flask import Flask

from . import config
from .openaccess import WorkflowOpenaccessService, WorkflowOpenaccessServiceConfig
from .theses import WorkflowThesesService, WorkflowThesesServiceConfig


class InvenioWorkflowsTugraz:
    """invenio-workflows-tugraz extension."""

    def __init__(self, app: Flask = None) -> None:
        """Extension initialization."""
        if app:
            self.init_app(app)

    def init_app(self, app: Flask) -> None:
        """Flask application initialization."""
        self.init_config(app)
        self.init_services(app)
        app.extensions["invenio-workflows-tugraz"] = self

    def init_config(self, app: Flask) -> None:
        """Initialize configuration."""
        for k in dir(config):
            attr = getattr(config, k)
            unprefixed_config = k.replace("WORKFLOWS_", "")

            if k.startswith("WORKFLOWS_TUGRAZ_"):
                app.config.setdefault(k, attr)

            if unprefixed_config in app.config:
                match app.config[unprefixed_config]:
                    case list() as container:
                        container.extend(attr)
                    case dict() as container:
                        container.update(attr)
                    case _:
                        app.config[unprefixed_config] = attr

            elif k.startswith("WORKFLOWS_"):
                match attr:
                    case list():
                        app.config.setdefault(unprefixed_config, [])
                        app.config[unprefixed_config].extend(attr)
                    case dict():
                        app.config.setdefault(unprefixed_config, {})
                        app.config[unprefixed_config].update(attr)
                    case _:
                        app.config[unprefixed_config] = attr

    def init_services(self, app: Flask) -> None:
        """Init services."""
        theses_config = WorkflowThesesServiceConfig.build(app)
        self.theses_service = WorkflowThesesService(config=theses_config)

        openaccess_config = WorkflowOpenaccessServiceConfig.build(app)
        self.openaccess_service = WorkflowOpenaccessService(config=openaccess_config)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Service for theses workflow."""

from flask_principal import Identity
from invenio_records_resources.services.base import Service
from invenio_records_resources.services.uow import (
    RecordCommitOp,
    UnitOfWork,
    unit_of_work,
)

from .api import WorkflowTheses


class WorkflowThesesService(Service):
    """Workflow theses service."""

    @property
    def theses_cls(self) -> WorkflowTheses:
        """Theses cls."""
        return self.config.theses_cls

    @unit_of_work()
    def create(
        self,
        _: Identity,
        id_: str,
        cms_id: str,
        uow: UnitOfWork = None,
    ) -> None:
        """Create."""
        entry = self.theses_cls.create(id_, cms_id)
        uow.register(RecordCommitOp(entry))

    @unit_of_work()
    def set_ready_to(
        self,
        _: Identity,
        id_: str,
        state: str,
        uow: UnitOfWork = None,
    ) -> None:
        """Archive."""
        entry = self.theses_cls.resolve(id_)
        entry.set_ready_to(id_, state=state)
        uow.register(RecordCommitOp(entry))

    def get_ready_to(self, _: Identity, state: str) -> list[tuple[str, str]]:
        """Get archived."""
        return self.theses_cls.get_ready_to(state=state)

    @unit_of_work()
    def set_state(
        self,
        _: Identity,
        id_: str,
        state: str,
        uow: UnitOfWork = None,
    ) -> None:
        """Archive."""
        entry = self.theses_cls.resolve(id_)
        entry.set_state(state=state)
        uow.register(RecordCommitOp(entry))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it
# and/or modify it under the terms of the MIT License; see LICENSE
# file for more details.

"""Converter Module to facilitate conversion of metadata."""

import time

from flask_principal import Identity
from invenio_records_lom import current_records_lom
from invenio_records_lom.utils import (
    LOMDuplicateRecordError,
    LOMMetadata,
    check_about_duplicate,
)
from marshmallow.exceptions import ValidationError

from .visitor import IMOOXToLOM


def imoox_import_func(
    imoox_record: dict,
    identity: Identity,
    *,
    dry_run: bool = False,
) -> None:
    """Create and publish function."""
    course_code = imoox_record["attributes"]["courseCode"]
    try:
        check_about_duplicate(course_code, "imoox")
    except LOMDuplicateRecordError as error:
        msg = f"DRY_RUN {error}" if dry_run else str(error)
        raise RuntimeError(msg) from error

    if dry_run:
        msg = f"DRY_RUN imoox import success course_code: {course_code}"
        raise RuntimeError(msg)

    converter = IMOOXToLOM()
    lom_record = LOMMetadata()

    converter.convert(imoox_record, lom_record)

    data = {
        "access": {"record": "public", "files": "public"},
        "files": {"enabled": False},
        "metadata": lom_record.json,
        "resource_type": "link",
    }

    lom_service = current_records_lom.records_service
    try:
        draft = lom_service.create(data=data, identity=identity)

        # to prevent the race condition bug.
        # see https://github.com/inveniosoftware/invenio-rdm-records/issues/809
        time.sleep(0.5)

        return lom_service.publish(id_=draft.id, identity=identity)
    except ValidationError as error:
        msg = f"ValidationError courseCode: {course_code}, error: {error}"
        raise RuntimeError(msg) from error



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2024 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Package serves as a place for the workflows of the repository of the TU Graz."""

from .ext import InvenioWorkflowsTugraz
from .proxies import current_workflows_tugraz

__version__ = "0.10.4"

__all__ = (
    "InvenioWorkflowsTugraz",
    "__version__",
    "current_workflows_tugraz",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Convert from LOM to Marc21."""


from .workflow import oer_create_aggregator, oer_create_in_alma_func

__all__ = (
    "oer_create_aggregator",
    "oer_create_in_alma_func",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Configuration file."""


from invenio_i18n import gettext as _
from invenio_rdm_records.services.pids.providers import ExternalPIDProvider

from .imoox import imoox_import_func
from .oer import oer_create_aggregator, oer_create_in_alma_func
from .openaccess import (
    openaccess_filter,
    openaccess_import_func,
    openaccess_mark_as_exported_aggregator,
    openaccess_update_status_in_pure,
)
from .teachcenter import teachcenter_import_func
from .theses import (
    theses_create_aggregator,
    theses_create_func,
    theses_duplicate_func,
    theses_filter,
    theses_import_from_alma_func,
    theses_import_from_cms_func,
    theses_update_aggregator,
    theses_update_func,
)

WORKFLOWS_ALMA_REPOSITORY_RECORDS_IMPORT_FUNCS = {
    "theses": theses_import_from_alma_func,
}
""""""

WORKFLOWS_ALMA_REPOSITORY_RECORDS_UPDATE_AGGREGATORS = {
    "theses": theses_update_aggregator,
}
""""""

WORKFLOWS_ALMA_REPOSITORY_RECORDS_UPDATE_FUNCS = {
    "theses": theses_update_func,
}
""""""

WORKFLOWS_ALMA_ALMA_RECORDS_CREATE_FUNCS = {
    "theses": theses_create_func,
    "oer": oer_create_in_alma_func,
}
""""""

WORKFLOWS_ALMA_ALMA_RECORDS_CREATE_AGGREGATORS = {
    "theses": theses_create_aggregator,
    "oer": oer_create_aggregator,
}
""""""

WORKFLOWS_CAMPUSONLINE_THESES_FILTER = theses_filter()
""""""

WORKFLOWS_CAMPUSONLINE_IMPORT_FUNC = theses_import_from_cms_func
""""""

WORKFLOWS_CAMPUSONLINE_DUPLICATE_FUNC = theses_duplicate_func
""""""

WORKFLOWS_IMOOX_IMPORT_FUNC = imoox_import_func
""""""

WORKFLOWS_LOM_PERSISTENT_IDENTIFIER_PROVIDERS = [
    ExternalPIDProvider("moodle", "moodle", label=_("MOODLE ID")),
]
"""List of persistent identifier providers.

The values are added to the LOM_PERSISTENT_IDENTIFIER_PROVIDERS list.
"""

WORKFLOWS_LOM_PERSISTENT_IDENTIFIERS = {
    "moodle": {
        "providers": ["moodle"],
        "required": False,
        "label": _("MOODLE"),
    },
    "moodle_alternative": {
        "providers": ["moodle"],
        "required": False,
        "label": _("MOODLE"),
    },
}
"""Dict of persistent identifiers.

The values are added to the LOM_PERSISTENT_IDENTIFIERS dict.
"""

WORKFLOWS_MARC21_PERSISTENT_IDENTIFIER_PROVIDERS = [
    ExternalPIDProvider("pure", "pure", label="Pure ID"),
]
"""A list of configured persistent identifier providers for Marc21."""

WORKFLOWS_MARC21_PERSISTENT_IDENTIFIERS = {
    "pure": {
        "providers": ["pure"],
        "required": False,
        "label": "PURE",
    },
}
"""The configured persistent identifiers for records for marc21."""

WORKFLOWS_MOODLE_REPOSITORY_IMPORT_FUNC = teachcenter_import_func
""""""

WORKFLOWS_PURE_IMPORT_FUNC = openaccess_import_func
"""See corresponding varaible in invenio-pure."""

WORKFLOWS_PURE_FILTER_RECORDS = openaccess_filter()

WORKFLOWS_PURE_MARK_AS_EXPORTED_FUNC = openaccess_update_status_in_pure

WORKFLOWS_PURE_MARK_AS_EXPORTED_AGGREGATOR = openaccess_mark_as_exported_aggregator



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Models for theses workflow."""

from invenio_db import db
from invenio_records.models import Timestamp
from sqlalchemy import BOOLEAN


class WorkflowOpenaccessMetadata(db.Model, Timestamp):
    """Workflow theses."""

    __tablename__ = "workflows_openaccess"

    pid = db.Column(db.String(255), primary_key=True)

    pure_id = db.Column(db.String(255))

    imported_in_repo = db.Column(BOOLEAN, default=False)

    marked_as_exported = db.Column(BOOLEAN, default=False)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it
# and/or modify it under the terms of the MIT License; see LICENSE
# file for more details.

"""Imoox module."""

from .workflows import imoox_import_func

__all__ = ("imoox_import_func",)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2024 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""CLI for theses workflow."""


from click import group, secho
from flask.cli import with_appcontext
from invenio_access.permissions import system_identity

from ..proxies import current_workflows_tugraz
from ..types import Color


@group("theses")
def theses_group() -> None:
    """Theses group."""


@theses_group.command()
@with_appcontext
def ready_to_archive() -> None:
    """Ready to archive."""
    theses_service = current_workflows_tugraz.theses_service
    entries = theses_service.get_ready_to(system_identity, state="archive_in_cms")
    secho("ready to archive in cms", fg=Color.neutral)
    for entry in entries:
        secho(f"pid: {entry.pid}, cms_id: {entry.cms_id}", fg=Color.neutral)


@theses_group.command()
@with_appcontext
def ready_to_publish() -> None:
    """Ready to archive."""
    theses_service = current_workflows_tugraz.theses_service
    entries = theses_service.get_ready_to(system_identity, state="publish_in_cms")
    secho("ready to publish in cms", fg=Color.neutral)
    for entry in entries:
        secho(f"pid: {entry.pid}, cms_id: {entry.cms_id}", fg=Color.neutral)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Alembic create tables for openaccess workflow."""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "bf15edb5b389"
down_revision = None
branch_labels = ()
depends_on = "74b15cacffbc"


def upgrade() -> None:
    """Upgrade database."""
    op.create_table(
        "workflows_openaccess",
        sa.Column("created", sa.DateTime(), nullable=False),
        sa.Column("updated", sa.DateTime(), nullable=False),
        sa.Column(
            "pid",
            sa.String(255),
            nullable=False,
        ),
        sa.Column(
            "pure_id",
            sa.String(255),
            nullable=False,
        ),
        sa.Column(
            "imported_in_repo",
            sa.Boolean(),
            default=False,
        ),
        sa.Column(
            "marked_as_exported",
            sa.Boolean(),
            default=False,
        ),
        sa.PrimaryKeyConstraint("pid", name=op.f("pk_workflows_openaccess")),
    )


def downgrade() -> None:
    """Downgrade database."""
    op.drop_table("workflows_openaccess")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Open Access Workflow."""

from .config import WorkflowOpenaccessServiceConfig
from .service import WorkflowOpenaccessService
from .workflow import (
    openaccess_filter,
    openaccess_import_func,
    openaccess_mark_as_exported_aggregator,
    openaccess_update_status_in_pure,
)

__all__ = (
    "WorkflowOpenaccessService",
    "WorkflowOpenaccessServiceConfig",
    "openaccess_filter",
    "openaccess_import_func",
    "openaccess_mark_as_exported_aggregator",
    "openaccess_update_status_in_pure",
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Theses jobs."""

from invenio_jobs.jobs import JobType

from .tasks import status_arch, status_pub


class StatusArchJob(JobType):
    """Status arch job."""

    id = "status_arch"
    title = "Update Status Archive in Campusonline"
    description = "Update the status 'archive' in campusonline."

    task = status_arch


class StatusPubJob(JobType):
    """Status publication job."""

    id = "status_pub"
    title = "Update Status Publication in Campusonline"
    description = "Update the status 'publication' in campusonline."

    task = status_pub



# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2025 Graz University of Technology.
#
# invenio-workflows-tugraz is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Openaccess Workflow utils."""

from invenio_pure import URL
from invenio_records_marc21 import check_about_duplicate

from .types import PureId


@check_about_duplicate.register
def _(value: PureId) -> None:
    """Check about double pure id."""
    check_about_duplicate(str(value), value.category)


def access_type(electronic_version: dict) -> str:
    """Get Access type."""
    try:
        return electronic_version["accessType"]["uri"]
    except (KeyError, TypeError):
        return False


def license_type(electronic_version: dict) -> str:
    """Get license type."""
    try:
        return electronic_version["licenseType"]["uri"]
    except (KeyError, TypeError):
        return False


def extract_files(pure_record: dict) -> URL:
    """Extract file url."""

    def condition(item: dict) -> bool:
        condition_1 = (
            access_type(item) == "/dk/atira/pure/core/openaccesspermission/open"
        )
        condition_2 = "cc_by" in license_type(item)
        return condition_1 and condition_2

    files = []
    for electronic_version in pure_record["electronicVersions"]:
        try:
            if condition(electronic_version):
                files.append(electronic_version["file"])
        except (KeyError, TypeError):
            continue

    if len(files) == 0:
        msg = f"record: {pure_record["uuid"]} doesn't provide downloadable files"
        raise RuntimeError(msg)

    return files


def change_to_exported(pure_record: dict) -> dict:
    """Replace the keyword group."""
    replaced = False
    for keyword_group in pure_record["keywordGroups"]:
        if (
            keyword_group["logicalName"]
            == "dk/atira/pure/researchoutput/keywords/export2repo"
        ):
            replaced = True
            keyword_group["classifications"] = [
                {
                    "uri": "dk/atira/pure/researchoutput/keywords/export2repo/exported",
                    "term": {
                        "en_GB": "Exported",
                        "de_DE": "Exportiert",
                    },
                },
            ]

    # necessary if the keyword group does not show up in keywordGroups due to
    # pure configuration
    if not replaced:
        keyword_group_validated = {
            "typeDiscriminator": "ClassificationsKeywordGroup",
            "pureId": 76333629,
            "logicalName": "dk/atira/pure/researchoutput/keywords/export2repo",
            "name": {
                "en_GB": "Export to Repository",
                "de_DE": "Export ins Repository",
            },
            "classifications": [
                {
                    "uri": "dk/atira/pure/researchoutput/keywords/export2repo/validated",
                    "term": {
                        "en_GB": "Validated",
                        "de_DE": "Validiert",
                    },
                },
                {
                    "uri": "dk/atira/pure/researchoutput/keywords/export2repo/exported",
                    "term": {
                        "en_GB": "Exported",
                        "de_DE": "Exportiert",
                    },
                },
            ],
        }
        pure_record["keywordGroups"].append(keyword_group_validated)
